// qstrs specific to this port

Q(help)
Q(input)
Q(collections)
Q(struct)

Q(microbit)
Q(reset)
Q(sleep)
Q(running_time)
Q(panic)
Q(temperature)

Q(this)
Q(authors)

Q(antigravity)

Q(love)
Q(badaboom)

Q(MicroBitDigitalPin)
Q(MicroBitAnalogDigitalPin)
Q(MicroBitTouchPin)
Q(read_digital)
Q(write_digital)
Q(read_analog)
Q(write_analog)
Q(set_analog_period)
Q(set_analog_period_microseconds)
Q(get_analog_period_microseconds)
Q(is_touched)
Q(unused)
Q(audio_play)
Q(button)
Q(touch)
Q(3v)
Q(get_mode)

Q(MicroBitIO)
Q(pin0)
Q(pin1)
Q(pin2)
Q(pin3)
Q(pin4)
Q(pin5)
Q(pin6)
Q(pin7)
Q(pin8)
Q(pin9)
Q(pin10)
Q(pin11)
Q(pin12)
Q(pin13)
Q(pin14)
Q(pin15)
Q(pin16)
Q(pin19)
Q(pin20)
Q(get_pull)
Q(set_pull)
Q(PULL_UP)
Q(PULL_DOWN)
Q(NO_PULL)

Q(MicroBitImage)
Q(Image)
Q(image)
Q(width)
Q(height)
Q(invert)
Q(fill)
Q(set_pixel)
Q(get_pixel)
Q(shift_left)
Q(shift_right)
Q(shift_up)
Q(shift_down)
Q(monospace)
Q(blit)
Q(HEART)
Q(HEART_SMALL)
Q(HAPPY)
Q(SAD)
Q(SMILE)
Q(CONFUSED)
Q(ANGRY)
Q(ASLEEP)
Q(SURPRISED)
Q(SILLY)
Q(FABULOUS)
Q(MEH)
Q(YES)
Q(NO)
Q(CLOCK12)
Q(CLOCK1)
Q(CLOCK2)
Q(CLOCK3)
Q(CLOCK4)
Q(CLOCK5)
Q(CLOCK6)
Q(CLOCK7)
Q(CLOCK8)
Q(CLOCK9)
Q(CLOCK10)
Q(CLOCK11)
Q(ARROW_N)
Q(ARROW_NE)
Q(ARROW_E)
Q(ARROW_SE)
Q(ARROW_S)
Q(ARROW_SW)
Q(ARROW_W)
Q(ARROW_NW)
Q(TRIANGLE)
Q(TRIANGLE_LEFT)
Q(CHESSBOARD)
Q(DIAMOND)
Q(DIAMOND_SMALL)
Q(SQUARE)
Q(SQUARE_SMALL)
Q(RABBIT)
Q(COW)
Q(MUSIC_CROTCHET)
Q(MUSIC_QUAVER)
Q(MUSIC_QUAVERS)
Q(PITCHFORK)
Q(XMAS)
Q(PACMAN)
Q(TARGET)
Q(TSHIRT)
Q(ROLLERSKATE)
Q(DUCK)
Q(HOUSE)
Q(TORTOISE)
Q(BUTTERFLY)
Q(STICKFIGURE)
Q(GHOST)
Q(SWORD)
Q(GIRAFFE)
Q(SKULL)
Q(UMBRELLA)
Q(SNAKE)

Q(ALL_ARROWS)
Q(ALL_CLOCKS)

Q(MicroBitDisplay)
Q(set_brightness)
Q(set_display_mode)
Q(display)
Q(show)
Q(scroll)
Q(delay)
Q(stride)
Q(start)
Q(wait)
Q(loop)
Q(copy)
Q(crop)
Q(slice)
Q(text)
Q(SlicedImage)
Q(ScrollingString)
Q(on)
Q(off)
Q(is_on)
Q(Facade)

Q(MicroBitButton)
Q(button_a)
Q(button_b)
Q(is_pressed)
Q(was_pressed)
Q(get_presses)

Q(MicroBitAccelerometer)
Q(accelerometer)
Q(get_x)
Q(get_y)
Q(get_z)
Q(get_values)
Q(current_gesture)
Q(is_gesture)
Q(was_gesture)
Q(get_gestures)
Q(up)
Q(down)
Q(left)
Q(right)
Q(face up)
Q(face down)
Q(freefall)
Q(3g)
Q(6g)
Q(8g)
Q(shake)

Q(MicroBitCompass)
Q(compass)
Q(heading)
Q(is_calibrated)
Q(calibrate)
Q(clear_calibration)
Q(get_x)
Q(get_y)
Q(get_z)
Q(get_field_strength)

Q(MicroBitI2C)
Q(i2c)
Q(read)
Q(write)
Q(addr)
Q(n)
Q(buf)
Q(repeat)
Q(freq)
Q(sda)
Q(scl)

Q(music)
Q(frequency)
Q(duration)
Q(pitch)
Q(pin)
Q(play)
Q(set_tempo)
Q(get_tempo)
Q(bpm)
Q(ticks)
Q(BADDY)
Q(BA_DING)
Q(BIRTHDAY)
Q(BLUES)
Q(CHASE)
Q(DADADADUM)
Q(ENTERTAINER)
Q(FUNERAL)
Q(FUNK)
Q(JUMP_DOWN)
Q(JUMP_UP)
Q(NYAN)
Q(ODE)
Q(POWER_DOWN)
Q(POWER_UP)
Q(PRELUDE)
Q(PUNCHLINE)
Q(PYTHON)
Q(RINGTONE)
Q(WAWAWAWAA)
Q(WEDDING)
Q(a)
Q(a#)
Q(a#:1)
Q(a#:3)
Q(a2)
Q(a4)
Q(a4:1)
Q(a4:3)
Q(a:1)
Q(a:2)
Q(a:4)
Q(a:5)
Q(b)
Q(b2:1)
Q(b3)
Q(b4)
Q(b4:1)
Q(b4:2)
Q(b5)
Q(b5:1)
Q(b:1)
Q(b:2)
Q(c)
Q(c#)
Q(c#5)
Q(c#5:1)
Q(c#5:2)
Q(c#:1)
Q(c#:8)
Q(c2:2)
Q(c3)
Q(c3:3)
Q(c3:4)
Q(c4)
Q(c4:1)
Q(c4:3)
Q(c4:4)
Q(c5)
Q(c5:1)
Q(c5:2)
Q(c5:3)
Q(c5:4)
Q(c:1)
Q(c:2)
Q(c:3)
Q(c:4)
Q(c:8)
Q(d)
Q(d#)
Q(d#5:2)
Q(d#:2)
Q(d#:3)
Q(d3)
Q(d4)
Q(d4:1)
Q(d5)
Q(d5:1)
Q(d5:2)
Q(d:1)
Q(d:2)
Q(d:3)
Q(d:4)
Q(d:5)
Q(d:6)
Q(d:8)
Q(e)
Q(e3:3)
Q(e4)
Q(e4:1)
Q(e5)
Q(e6:3)
Q(e:1)
Q(e:2)
Q(e:3)
Q(e:4)
Q(e:5)
Q(e:6)
Q(e:8)
Q(eb:8)
Q(f)
Q(f#)
Q(f#5)
Q(f#5:2)
Q(f#:1)
Q(f#:2)
Q(f#:8)
Q(f2)
Q(f:1)
Q(f:2)
Q(f:3)
Q(f:4)
Q(f:8)
Q(g)
Q(g#)
Q(g#:1)
Q(g#:3)
Q(g3:1)
Q(g4)
Q(g4:1)
Q(g4:2)
Q(g5)
Q(g5:1)
Q(g:1)
Q(g:2)
Q(g:3)
Q(g:8)
Q(r)
Q(r4:2)
Q(r:1)
Q(r:2)
Q(r:3)

Q(MicroBitUART)
Q(uart)
Q(init)
Q(baudrate)
Q(bits)
Q(parity)
Q(stop)
Q(pins)
Q(tx)
Q(rx)
Q(any)
Q(read)
Q(readall)
Q(readline)
Q(readinto)
Q(write)
Q(ODD)
Q(EVEN)

Q(MicroBitSPI)
Q(spi)
Q(init)
Q(baudrate)
Q(bits)
Q(mode)
Q(sclk)
Q(mosi)
Q(miso)
Q(write)
Q(write_readinto)

Q(neopixel)
Q(NeoPixel)
Q(clear)
Q(show)

Q(random)
Q(getrandbits)
Q(seed)
Q(randrange)
Q(randint)
Q(choice)
Q(uniform)

Q(audio)
Q(play)
Q(AudioFrame)
Q(pin)
Q(return_pin)
Q(source)
Q(copyfrom)

Q(name)

Q(os)
Q(uname)
Q(micropython)
Q(sysname)
Q(nodename)
Q(release)
Q(version)
Q(BytesIO)
Q(TextIO)
Q(read)
Q(write)
Q(writable)
Q(readall)
Q(name)
Q(listdir)
Q(machine)
Q(size)

Q(is_playing)

Q(speech)
Q(say)
Q(pronounce)
Q(sing)
Q(pitch)
Q(throat)
Q(mouth)
Q(speed)
Q(debug)
Q(translate)

Q(radio)
Q(reset)
Q(config)
Q(on)
Q(off)
Q(send_bytes)
Q(receive_bytes)
Q(send)
Q(receive)
Q(receive_bytes_into)
Q(receive_full)
Q(length)
Q(queue)
Q(channel)
Q(power)
Q(data_rate)
Q(address)
Q(group)
Q(RATE_250KBIT)
Q(RATE_1MBIT)
Q(RATE_2MBIT)
