// This file was automatically generated by makeqstrdata.py

QDEF(MP_QSTR_NULL, (const byte*)"\x00\x00" "")
QDEF(MP_QSTR_, (const byte*)"\x05\x00" "")
QDEF(MP_QSTR__star_, (const byte*)"\x8f\x01" "*")
QDEF(MP_QSTR__, (const byte*)"\xfa\x01" "_")
QDEF(MP_QSTR__slash_, (const byte*)"\x8a\x01" "/")
QDEF(MP_QSTR__percent__hash_o, (const byte*)"\x6c\x03" "%#o")
QDEF(MP_QSTR__percent__hash_x, (const byte*)"\x7b\x03" "%#x")
QDEF(MP_QSTR__brace_open__colon__hash_b_brace_close_, (const byte*)"\x58\x05" "{:#b}")
QDEF(MP_QSTR__0x0a_, (const byte*)"\xaf\x01" "\x0a")
QDEF(MP_QSTR_maximum_space_recursion_space_depth_space_exceeded, (const byte*)"\x73\x20" "maximum recursion depth exceeded")
QDEF(MP_QSTR__lt_module_gt_, (const byte*)"\xbd\x08" "<module>")
QDEF(MP_QSTR__lt_lambda_gt_, (const byte*)"\x80\x08" "<lambda>")
QDEF(MP_QSTR__lt_listcomp_gt_, (const byte*)"\xd4\x0a" "<listcomp>")
QDEF(MP_QSTR__lt_dictcomp_gt_, (const byte*)"\xcc\x0a" "<dictcomp>")
QDEF(MP_QSTR__lt_setcomp_gt_, (const byte*)"\x54\x09" "<setcomp>")
QDEF(MP_QSTR__lt_genexpr_gt_, (const byte*)"\x34\x09" "<genexpr>")
QDEF(MP_QSTR__lt_string_gt_, (const byte*)"\x52\x08" "<string>")
QDEF(MP_QSTR__lt_stdin_gt_, (const byte*)"\xe3\x07" "<stdin>")
QDEF(MP_QSTR_utf_hyphen_8, (const byte*)"\xb7\x05" "utf-8")
QDEF(MP_QSTR_help, (const byte*)"\x94\x04" "help")
QDEF(MP_QSTR_input, (const byte*)"\x73\x05" "input")
QDEF(MP_QSTR_collections, (const byte*)"\xe0\x0b" "collections")
QDEF(MP_QSTR_struct, (const byte*)"\x12\x06" "struct")
QDEF(MP_QSTR_microbit, (const byte*)"\xc0\x08" "microbit")
QDEF(MP_QSTR_reset, (const byte*)"\x10\x05" "reset")
QDEF(MP_QSTR_sleep, (const byte*)"\xea\x05" "sleep")
QDEF(MP_QSTR_running_time, (const byte*)"\xc8\x0c" "running_time")
QDEF(MP_QSTR_panic, (const byte*)"\xd0\x05" "panic")
QDEF(MP_QSTR_temperature, (const byte*)"\xe9\x0b" "temperature")
QDEF(MP_QSTR_this, (const byte*)"\xa3\x04" "this")
QDEF(MP_QSTR_authors, (const byte*)"\x63\x07" "authors")
QDEF(MP_QSTR_antigravity, (const byte*)"\xf1\x0b" "antigravity")
QDEF(MP_QSTR_love, (const byte*)"\x55\x04" "love")
QDEF(MP_QSTR_badaboom, (const byte*)"\x2c\x08" "badaboom")
QDEF(MP_QSTR_MicroBitDigitalPin, (const byte*)"\xcd\x12" "MicroBitDigitalPin")
QDEF(MP_QSTR_MicroBitAnalogDigitalPin, (const byte*)"\x07\x18" "MicroBitAnalogDigitalPin")
QDEF(MP_QSTR_MicroBitTouchPin, (const byte*)"\x52\x10" "MicroBitTouchPin")
QDEF(MP_QSTR_read_digital, (const byte*)"\x92\x0c" "read_digital")
QDEF(MP_QSTR_write_digital, (const byte*)"\xfd\x0d" "write_digital")
QDEF(MP_QSTR_read_analog, (const byte*)"\x62\x0b" "read_analog")
QDEF(MP_QSTR_write_analog, (const byte*)"\x2d\x0c" "write_analog")
QDEF(MP_QSTR_set_analog_period, (const byte*)"\x08\x11" "set_analog_period")
QDEF(MP_QSTR_set_analog_period_microseconds, (const byte*)"\xee\x1e" "set_analog_period_microseconds")
QDEF(MP_QSTR_get_analog_period_microseconds, (const byte*)"\x7a\x1e" "get_analog_period_microseconds")
QDEF(MP_QSTR_is_touched, (const byte*)"\x04\x0a" "is_touched")
QDEF(MP_QSTR_unused, (const byte*)"\x79\x06" "unused")
QDEF(MP_QSTR_audio_play, (const byte*)"\xc8\x0a" "audio_play")
QDEF(MP_QSTR_button, (const byte*)"\xf3\x06" "button")
QDEF(MP_QSTR_touch, (const byte*)"\x80\x05" "touch")
QDEF(MP_QSTR_3v, (const byte*)"\x20\x02" "3v")
QDEF(MP_QSTR_get_mode, (const byte*)"\xaf\x08" "get_mode")
QDEF(MP_QSTR_MicroBitIO, (const byte*)"\xe6\x0a" "MicroBitIO")
QDEF(MP_QSTR_pin0, (const byte*)"\x02\x04" "pin0")
QDEF(MP_QSTR_pin1, (const byte*)"\x03\x04" "pin1")
QDEF(MP_QSTR_pin2, (const byte*)"\x01\x04" "pin2")
QDEF(MP_QSTR_pin3, (const byte*)"\x01\x04" "pin3")
QDEF(MP_QSTR_pin4, (const byte*)"\x06\x04" "pin4")
QDEF(MP_QSTR_pin5, (const byte*)"\x07\x04" "pin5")
QDEF(MP_QSTR_pin6, (const byte*)"\x04\x04" "pin6")
QDEF(MP_QSTR_pin7, (const byte*)"\x05\x04" "pin7")
QDEF(MP_QSTR_pin8, (const byte*)"\x0a\x04" "pin8")
QDEF(MP_QSTR_pin9, (const byte*)"\x0b\x04" "pin9")
QDEF(MP_QSTR_pin10, (const byte*)"\x53\x05" "pin10")
QDEF(MP_QSTR_pin11, (const byte*)"\x52\x05" "pin11")
QDEF(MP_QSTR_pin12, (const byte*)"\x51\x05" "pin12")
QDEF(MP_QSTR_pin13, (const byte*)"\x50\x05" "pin13")
QDEF(MP_QSTR_pin14, (const byte*)"\x57\x05" "pin14")
QDEF(MP_QSTR_pin15, (const byte*)"\x56\x05" "pin15")
QDEF(MP_QSTR_pin16, (const byte*)"\x55\x05" "pin16")
QDEF(MP_QSTR_pin19, (const byte*)"\x5a\x05" "pin19")
QDEF(MP_QSTR_pin20, (const byte*)"\x30\x05" "pin20")
QDEF(MP_QSTR_get_pull, (const byte*)"\x49\x08" "get_pull")
QDEF(MP_QSTR_set_pull, (const byte*)"\xdd\x08" "set_pull")
QDEF(MP_QSTR_PULL_UP, (const byte*)"\xba\x07" "PULL_UP")
QDEF(MP_QSTR_PULL_DOWN, (const byte*)"\xad\x09" "PULL_DOWN")
QDEF(MP_QSTR_NO_PULL, (const byte*)"\x1e\x07" "NO_PULL")
QDEF(MP_QSTR_MicroBitImage, (const byte*)"\x87\x0d" "MicroBitImage")
QDEF(MP_QSTR_Image, (const byte*)"\x62\x05" "Image")
QDEF(MP_QSTR_image, (const byte*)"\x42\x05" "image")
QDEF(MP_QSTR_width, (const byte*)"\x23\x05" "width")
QDEF(MP_QSTR_height, (const byte*)"\xfa\x06" "height")
QDEF(MP_QSTR_invert, (const byte*)"\xb7\x06" "invert")
QDEF(MP_QSTR_fill, (const byte*)"\xca\x04" "fill")
QDEF(MP_QSTR_set_pixel, (const byte*)"\xb0\x09" "set_pixel")
QDEF(MP_QSTR_get_pixel, (const byte*)"\xa4\x09" "get_pixel")
QDEF(MP_QSTR_shift_left, (const byte*)"\xa1\x0a" "shift_left")
QDEF(MP_QSTR_shift_right, (const byte*)"\xba\x0b" "shift_right")
QDEF(MP_QSTR_shift_up, (const byte*)"\xdf\x08" "shift_up")
QDEF(MP_QSTR_shift_down, (const byte*)"\x48\x0a" "shift_down")
QDEF(MP_QSTR_monospace, (const byte*)"\xe2\x09" "monospace")
QDEF(MP_QSTR_blit, (const byte*)"\xf6\x04" "blit")
QDEF(MP_QSTR_HEART, (const byte*)"\x0f\x05" "HEART")
QDEF(MP_QSTR_HEART_SMALL, (const byte*)"\xcf\x0b" "HEART_SMALL")
QDEF(MP_QSTR_HAPPY, (const byte*)"\x15\x05" "HAPPY")
QDEF(MP_QSTR_SAD, (const byte*)"\x93\x03" "SAD")
QDEF(MP_QSTR_SMILE, (const byte*)"\x9b\x05" "SMILE")
QDEF(MP_QSTR_CONFUSED, (const byte*)"\xa6\x08" "CONFUSED")
QDEF(MP_QSTR_ANGRY, (const byte*)"\x26\x05" "ANGRY")
QDEF(MP_QSTR_ASLEEP, (const byte*)"\x0b\x06" "ASLEEP")
QDEF(MP_QSTR_SURPRISED, (const byte*)"\x88\x09" "SURPRISED")
QDEF(MP_QSTR_SILLY, (const byte*)"\xc6\x05" "SILLY")
QDEF(MP_QSTR_FABULOUS, (const byte*)"\x30\x08" "FABULOUS")
QDEF(MP_QSTR_MEH, (const byte*)"\x05\x03" "MEH")
QDEF(MP_QSTR_YES, (const byte*)"\x0a\x03" "YES")
QDEF(MP_QSTR_NO, (const byte*)"\x04\x02" "NO")
QDEF(MP_QSTR_CLOCK12, (const byte*)"\x6e\x07" "CLOCK12")
QDEF(MP_QSTR_CLOCK1, (const byte*)"\xdc\x06" "CLOCK1")
QDEF(MP_QSTR_CLOCK2, (const byte*)"\xdf\x06" "CLOCK2")
QDEF(MP_QSTR_CLOCK3, (const byte*)"\xde\x06" "CLOCK3")
QDEF(MP_QSTR_CLOCK4, (const byte*)"\xd9\x06" "CLOCK4")
QDEF(MP_QSTR_CLOCK5, (const byte*)"\xd8\x06" "CLOCK5")
QDEF(MP_QSTR_CLOCK6, (const byte*)"\xdb\x06" "CLOCK6")
QDEF(MP_QSTR_CLOCK7, (const byte*)"\xda\x06" "CLOCK7")
QDEF(MP_QSTR_CLOCK8, (const byte*)"\xd5\x06" "CLOCK8")
QDEF(MP_QSTR_CLOCK9, (const byte*)"\xd4\x06" "CLOCK9")
QDEF(MP_QSTR_CLOCK10, (const byte*)"\x6c\x07" "CLOCK10")
QDEF(MP_QSTR_CLOCK11, (const byte*)"\x6d\x07" "CLOCK11")
QDEF(MP_QSTR_ARROW_N, (const byte*)"\xed\x07" "ARROW_N")
QDEF(MP_QSTR_ARROW_NE, (const byte*)"\xc8\x08" "ARROW_NE")
QDEF(MP_QSTR_ARROW_E, (const byte*)"\xe6\x07" "ARROW_E")
QDEF(MP_QSTR_ARROW_SE, (const byte*)"\xb5\x08" "ARROW_SE")
QDEF(MP_QSTR_ARROW_S, (const byte*)"\xf0\x07" "ARROW_S")
QDEF(MP_QSTR_ARROW_SW, (const byte*)"\xa7\x08" "ARROW_SW")
QDEF(MP_QSTR_ARROW_W, (const byte*)"\xf4\x07" "ARROW_W")
QDEF(MP_QSTR_ARROW_NW, (const byte*)"\xda\x08" "ARROW_NW")
QDEF(MP_QSTR_TRIANGLE, (const byte*)"\xeb\x08" "TRIANGLE")
QDEF(MP_QSTR_TRIANGLE_LEFT, (const byte*)"\x8f\x0d" "TRIANGLE_LEFT")
QDEF(MP_QSTR_CHESSBOARD, (const byte*)"\x51\x0a" "CHESSBOARD")
QDEF(MP_QSTR_DIAMOND, (const byte*)"\xa1\x07" "DIAMOND")
QDEF(MP_QSTR_DIAMOND_SMALL, (const byte*)"\x61\x0d" "DIAMOND_SMALL")
QDEF(MP_QSTR_SQUARE, (const byte*)"\x84\x06" "SQUARE")
QDEF(MP_QSTR_SQUARE_SMALL, (const byte*)"\x84\x0c" "SQUARE_SMALL")
QDEF(MP_QSTR_RABBIT, (const byte*)"\xeb\x06" "RABBIT")
QDEF(MP_QSTR_COW, (const byte*)"\x5e\x03" "COW")
QDEF(MP_QSTR_MUSIC_CROTCHET, (const byte*)"\xab\x0e" "MUSIC_CROTCHET")
QDEF(MP_QSTR_MUSIC_QUAVER, (const byte*)"\x5f\x0c" "MUSIC_QUAVER")
QDEF(MP_QSTR_MUSIC_QUAVERS, (const byte*)"\x6c\x0d" "MUSIC_QUAVERS")
QDEF(MP_QSTR_PITCHFORK, (const byte*)"\x33\x09" "PITCHFORK")
QDEF(MP_QSTR_XMAS, (const byte*)"\xe2\x04" "XMAS")
QDEF(MP_QSTR_PACMAN, (const byte*)"\x55\x06" "PACMAN")
QDEF(MP_QSTR_TARGET, (const byte*)"\xb4\x06" "TARGET")
QDEF(MP_QSTR_TSHIRT, (const byte*)"\x45\x06" "TSHIRT")
QDEF(MP_QSTR_ROLLERSKATE, (const byte*)"\x27\x0b" "ROLLERSKATE")
QDEF(MP_QSTR_DUCK, (const byte*)"\x3c\x04" "DUCK")
QDEF(MP_QSTR_HOUSE, (const byte*)"\xa1\x05" "HOUSE")
QDEF(MP_QSTR_TORTOISE, (const byte*)"\xc8\x08" "TORTOISE")
QDEF(MP_QSTR_BUTTERFLY, (const byte*)"\x56\x09" "BUTTERFLY")
QDEF(MP_QSTR_STICKFIGURE, (const byte*)"\xe9\x0b" "STICKFIGURE")
QDEF(MP_QSTR_GHOST, (const byte*)"\xc2\x05" "GHOST")
QDEF(MP_QSTR_SWORD, (const byte*)"\x98\x05" "SWORD")
QDEF(MP_QSTR_GIRAFFE, (const byte*)"\x7d\x07" "GIRAFFE")
QDEF(MP_QSTR_SKULL, (const byte*)"\x48\x05" "SKULL")
QDEF(MP_QSTR_UMBRELLA, (const byte*)"\xc9\x08" "UMBRELLA")
QDEF(MP_QSTR_SNAKE, (const byte*)"\x97\x05" "SNAKE")
QDEF(MP_QSTR_ALL_ARROWS, (const byte*)"\xf1\x0a" "ALL_ARROWS")
QDEF(MP_QSTR_ALL_CLOCKS, (const byte*)"\xc0\x0a" "ALL_CLOCKS")
QDEF(MP_QSTR_MicroBitDisplay, (const byte*)"\x5a\x0f" "MicroBitDisplay")
QDEF(MP_QSTR_set_brightness, (const byte*)"\x91\x0e" "set_brightness")
QDEF(MP_QSTR_set_display_mode, (const byte*)"\x9e\x10" "set_display_mode")
QDEF(MP_QSTR_display, (const byte*)"\x1f\x07" "display")
QDEF(MP_QSTR_show, (const byte*)"\x86\x04" "show")
QDEF(MP_QSTR_scroll, (const byte*)"\x28\x06" "scroll")
QDEF(MP_QSTR_delay, (const byte*)"\x50\x05" "delay")
QDEF(MP_QSTR_stride, (const byte*)"\xb8\x06" "stride")
QDEF(MP_QSTR_start, (const byte*)"\x85\x05" "start")
QDEF(MP_QSTR_wait, (const byte*)"\x8e\x04" "wait")
QDEF(MP_QSTR_loop, (const byte*)"\x39\x04" "loop")
QDEF(MP_QSTR_copy, (const byte*)"\xe0\x04" "copy")
QDEF(MP_QSTR_crop, (const byte*)"\x0b\x04" "crop")
QDEF(MP_QSTR_slice, (const byte*)"\xb5\x05" "slice")
QDEF(MP_QSTR_text, (const byte*)"\x98\x04" "text")
QDEF(MP_QSTR_SlicedImage, (const byte*)"\xf6\x0b" "SlicedImage")
QDEF(MP_QSTR_ScrollingString, (const byte*)"\xbd\x0f" "ScrollingString")
QDEF(MP_QSTR_on, (const byte*)"\x64\x02" "on")
QDEF(MP_QSTR_off, (const byte*)"\x8a\x03" "off")
QDEF(MP_QSTR_is_on, (const byte*)"\x61\x05" "is_on")
QDEF(MP_QSTR_Facade, (const byte*)"\xc1\x06" "Facade")
QDEF(MP_QSTR_MicroBitButton, (const byte*)"\x16\x0e" "MicroBitButton")
QDEF(MP_QSTR_button_a, (const byte*)"\xed\x08" "button_a")
QDEF(MP_QSTR_button_b, (const byte*)"\xee\x08" "button_b")
QDEF(MP_QSTR_is_pressed, (const byte*)"\xe6\x0a" "is_pressed")
QDEF(MP_QSTR_was_pressed, (const byte*)"\xf9\x0b" "was_pressed")
QDEF(MP_QSTR_get_presses, (const byte*)"\xfd\x0b" "get_presses")
QDEF(MP_QSTR_MicroBitAccelerometer, (const byte*)"\x5b\x15" "MicroBitAccelerometer")
QDEF(MP_QSTR_accelerometer, (const byte*)"\x1e\x0d" "accelerometer")
QDEF(MP_QSTR_get_x, (const byte*)"\x34\x05" "get_x")
QDEF(MP_QSTR_get_y, (const byte*)"\x35\x05" "get_y")
QDEF(MP_QSTR_get_z, (const byte*)"\x36\x05" "get_z")
QDEF(MP_QSTR_get_values, (const byte*)"\xf4\x0a" "get_values")
QDEF(MP_QSTR_current_gesture, (const byte*)"\xd4\x0f" "current_gesture")
QDEF(MP_QSTR_is_gesture, (const byte*)"\x07\x0a" "is_gesture")
QDEF(MP_QSTR_was_gesture, (const byte*)"\xd8\x0b" "was_gesture")
QDEF(MP_QSTR_get_gestures, (const byte*)"\x18\x0c" "get_gestures")
QDEF(MP_QSTR_up, (const byte*)"\xa0\x02" "up")
QDEF(MP_QSTR_down, (const byte*)"\x37\x04" "down")
QDEF(MP_QSTR_left, (const byte*)"\xde\x04" "left")
QDEF(MP_QSTR_right, (const byte*)"\xe5\x05" "right")
QDEF(MP_QSTR_face_space_up, (const byte*)"\x21\x07" "face up")
QDEF(MP_QSTR_face_space_down, (const byte*)"\x36\x09" "face down")
QDEF(MP_QSTR_freefall, (const byte*)"\xb6\x08" "freefall")
QDEF(MP_QSTR_3g, (const byte*)"\x31\x02" "3g")
QDEF(MP_QSTR_6g, (const byte*)"\x94\x02" "6g")
QDEF(MP_QSTR_8g, (const byte*)"\x5a\x02" "8g")
QDEF(MP_QSTR_shake, (const byte*)"\x31\x05" "shake")
QDEF(MP_QSTR_MicroBitCompass, (const byte*)"\x10\x0f" "MicroBitCompass")
QDEF(MP_QSTR_compass, (const byte*)"\x55\x07" "compass")
QDEF(MP_QSTR_heading, (const byte*)"\x2d\x07" "heading")
QDEF(MP_QSTR_is_calibrated, (const byte*)"\x23\x0d" "is_calibrated")
QDEF(MP_QSTR_calibrate, (const byte*)"\x02\x09" "calibrate")
QDEF(MP_QSTR_clear_calibration, (const byte*)"\x49\x11" "clear_calibration")
QDEF(MP_QSTR_get_field_strength, (const byte*)"\xf4\x12" "get_field_strength")
QDEF(MP_QSTR_MicroBitI2C, (const byte*)"\xb8\x0b" "MicroBitI2C")
QDEF(MP_QSTR_i2c, (const byte*)"\x5d\x03" "i2c")
QDEF(MP_QSTR_read, (const byte*)"\xb7\x04" "read")
QDEF(MP_QSTR_write, (const byte*)"\x98\x05" "write")
QDEF(MP_QSTR_addr, (const byte*)"\xb6\x04" "addr")
QDEF(MP_QSTR_n, (const byte*)"\xcb\x01" "n")
QDEF(MP_QSTR_buf, (const byte*)"\x74\x03" "buf")
QDEF(MP_QSTR_repeat, (const byte*)"\xf2\x06" "repeat")
QDEF(MP_QSTR_freq, (const byte*)"\xe5\x04" "freq")
QDEF(MP_QSTR_sda, (const byte*)"\x53\x03" "sda")
QDEF(MP_QSTR_scl, (const byte*)"\xf9\x03" "scl")
QDEF(MP_QSTR_music, (const byte*)"\x04\x05" "music")
QDEF(MP_QSTR_frequency, (const byte*)"\xa1\x09" "frequency")
QDEF(MP_QSTR_duration, (const byte*)"\x7b\x08" "duration")
QDEF(MP_QSTR_pitch, (const byte*)"\x83\x05" "pitch")
QDEF(MP_QSTR_pin, (const byte*)"\xf2\x03" "pin")
QDEF(MP_QSTR_play, (const byte*)"\x21\x04" "play")
QDEF(MP_QSTR_set_tempo, (const byte*)"\x9b\x09" "set_tempo")
QDEF(MP_QSTR_get_tempo, (const byte*)"\x8f\x09" "get_tempo")
QDEF(MP_QSTR_bpm, (const byte*)"\xda\x03" "bpm")
QDEF(MP_QSTR_ticks, (const byte*)"\x43\x05" "ticks")
QDEF(MP_QSTR_BADDY, (const byte*)"\x9f\x05" "BADDY")
QDEF(MP_QSTR_BA_DING, (const byte*)"\x9d\x07" "BA_DING")
QDEF(MP_QSTR_BIRTHDAY, (const byte*)"\xfc\x08" "BIRTHDAY")
QDEF(MP_QSTR_BLUES, (const byte*)"\xc8\x05" "BLUES")
QDEF(MP_QSTR_CHASE, (const byte*)"\x59\x05" "CHASE")
QDEF(MP_QSTR_DADADADUM, (const byte*)"\xfc\x09" "DADADADUM")
QDEF(MP_QSTR_ENTERTAINER, (const byte*)"\x48\x0b" "ENTERTAINER")
QDEF(MP_QSTR_FUNERAL, (const byte*)"\x42\x07" "FUNERAL")
QDEF(MP_QSTR_FUNK, (const byte*)"\xd3\x04" "FUNK")
QDEF(MP_QSTR_JUMP_DOWN, (const byte*)"\xaa\x09" "JUMP_DOWN")
QDEF(MP_QSTR_JUMP_UP, (const byte*)"\xfd\x07" "JUMP_UP")
QDEF(MP_QSTR_NYAN, (const byte*)"\x3d\x04" "NYAN")
QDEF(MP_QSTR_ODE, (const byte*)"\x6b\x03" "ODE")
QDEF(MP_QSTR_POWER_DOWN, (const byte*)"\x97\x0a" "POWER_DOWN")
QDEF(MP_QSTR_POWER_UP, (const byte*)"\x01\x08" "POWER_UP")
QDEF(MP_QSTR_PRELUDE, (const byte*)"\x3a\x07" "PRELUDE")
QDEF(MP_QSTR_PUNCHLINE, (const byte*)"\xeb\x09" "PUNCHLINE")
QDEF(MP_QSTR_PYTHON, (const byte*)"\xb1\x06" "PYTHON")
QDEF(MP_QSTR_RINGTONE, (const byte*)"\xc7\x08" "RINGTONE")
QDEF(MP_QSTR_WAWAWAWAA, (const byte*)"\x64\x09" "WAWAWAWAA")
QDEF(MP_QSTR_WEDDING, (const byte*)"\x17\x07" "WEDDING")
QDEF(MP_QSTR_a, (const byte*)"\xc4\x01" "a")
QDEF(MP_QSTR_a_hash_, (const byte*)"\x67\x02" "a#")
QDEF(MP_QSTR_a_hash__colon_1, (const byte*)"\x2c\x04" "a#:1")
QDEF(MP_QSTR_a_hash__colon_3, (const byte*)"\x2e\x04" "a#:3")
QDEF(MP_QSTR_a2, (const byte*)"\x76\x02" "a2")
QDEF(MP_QSTR_a4, (const byte*)"\x70\x02" "a4")
QDEF(MP_QSTR_a4_colon_1, (const byte*)"\xbb\x04" "a4:1")
QDEF(MP_QSTR_a4_colon_3, (const byte*)"\xb9\x04" "a4:3")
QDEF(MP_QSTR_a_colon_1, (const byte*)"\x0f\x03" "a:1")
QDEF(MP_QSTR_a_colon_2, (const byte*)"\x0c\x03" "a:2")
QDEF(MP_QSTR_a_colon_4, (const byte*)"\x0a\x03" "a:4")
QDEF(MP_QSTR_a_colon_5, (const byte*)"\x0b\x03" "a:5")
QDEF(MP_QSTR_b, (const byte*)"\xc7\x01" "b")
QDEF(MP_QSTR_b2_colon_1, (const byte*)"\xde\x04" "b2:1")
QDEF(MP_QSTR_b3, (const byte*)"\x94\x02" "b3")
QDEF(MP_QSTR_b4, (const byte*)"\x93\x02" "b4")
QDEF(MP_QSTR_b4_colon_1, (const byte*)"\xd8\x04" "b4:1")
QDEF(MP_QSTR_b4_colon_2, (const byte*)"\xdb\x04" "b4:2")
QDEF(MP_QSTR_b5, (const byte*)"\x92\x02" "b5")
QDEF(MP_QSTR_b5_colon_1, (const byte*)"\xd9\x04" "b5:1")
QDEF(MP_QSTR_b_colon_1, (const byte*)"\x0c\x03" "b:1")
QDEF(MP_QSTR_b_colon_2, (const byte*)"\x0f\x03" "b:2")
QDEF(MP_QSTR_c, (const byte*)"\xc6\x01" "c")
QDEF(MP_QSTR_c_hash_, (const byte*)"\xa5\x02" "c#")
QDEF(MP_QSTR_c_hash_5, (const byte*)"\x70\x03" "c#5")
QDEF(MP_QSTR_c_hash_5_colon_1, (const byte*)"\xbb\x05" "c#5:1")
QDEF(MP_QSTR_c_hash_5_colon_2, (const byte*)"\xb8\x05" "c#5:2")
QDEF(MP_QSTR_c_hash__colon_1, (const byte*)"\x6e\x04" "c#:1")
QDEF(MP_QSTR_c_hash__colon_8, (const byte*)"\x67\x04" "c#:8")
QDEF(MP_QSTR_c2_colon_2, (const byte*)"\xfc\x04" "c2:2")
QDEF(MP_QSTR_c3, (const byte*)"\xb5\x02" "c3")
QDEF(MP_QSTR_c3_colon_3, (const byte*)"\x7c\x04" "c3:3")
QDEF(MP_QSTR_c3_colon_4, (const byte*)"\x7b\x04" "c3:4")
QDEF(MP_QSTR_c4, (const byte*)"\xb2\x02" "c4")
QDEF(MP_QSTR_c4_colon_1, (const byte*)"\xf9\x04" "c4:1")
QDEF(MP_QSTR_c4_colon_3, (const byte*)"\xfb\x04" "c4:3")
QDEF(MP_QSTR_c4_colon_4, (const byte*)"\xfc\x04" "c4:4")
QDEF(MP_QSTR_c5, (const byte*)"\xb3\x02" "c5")
QDEF(MP_QSTR_c5_colon_1, (const byte*)"\x78\x04" "c5:1")
QDEF(MP_QSTR_c5_colon_2, (const byte*)"\x7b\x04" "c5:2")
QDEF(MP_QSTR_c5_colon_3, (const byte*)"\x7a\x04" "c5:3")
QDEF(MP_QSTR_c5_colon_4, (const byte*)"\x7d\x04" "c5:4")
QDEF(MP_QSTR_c_colon_1, (const byte*)"\x0d\x03" "c:1")
QDEF(MP_QSTR_c_colon_2, (const byte*)"\x0e\x03" "c:2")
QDEF(MP_QSTR_c_colon_3, (const byte*)"\x0f\x03" "c:3")
QDEF(MP_QSTR_c_colon_4, (const byte*)"\x08\x03" "c:4")
QDEF(MP_QSTR_c_colon_8, (const byte*)"\x04\x03" "c:8")
QDEF(MP_QSTR_d, (const byte*)"\xc1\x01" "d")
QDEF(MP_QSTR_d_hash_, (const byte*)"\xc2\x02" "d#")
QDEF(MP_QSTR_d_hash_5_colon_2, (const byte*)"\xff\x05" "d#5:2")
QDEF(MP_QSTR_d_hash__colon_2, (const byte*)"\x0a\x04" "d#:2")
QDEF(MP_QSTR_d_hash__colon_3, (const byte*)"\x0b\x04" "d#:3")
QDEF(MP_QSTR_d3, (const byte*)"\xd2\x02" "d3")
QDEF(MP_QSTR_d4, (const byte*)"\xd5\x02" "d4")
QDEF(MP_QSTR_d4_colon_1, (const byte*)"\x1e\x04" "d4:1")
QDEF(MP_QSTR_d5, (const byte*)"\xd4\x02" "d5")
QDEF(MP_QSTR_d5_colon_1, (const byte*)"\x1f\x04" "d5:1")
QDEF(MP_QSTR_d5_colon_2, (const byte*)"\x1c\x04" "d5:2")
QDEF(MP_QSTR_d_colon_1, (const byte*)"\x0a\x03" "d:1")
QDEF(MP_QSTR_d_colon_2, (const byte*)"\x09\x03" "d:2")
QDEF(MP_QSTR_d_colon_3, (const byte*)"\x08\x03" "d:3")
QDEF(MP_QSTR_d_colon_4, (const byte*)"\x0f\x03" "d:4")
QDEF(MP_QSTR_d_colon_5, (const byte*)"\x0e\x03" "d:5")
QDEF(MP_QSTR_d_colon_6, (const byte*)"\x0d\x03" "d:6")
QDEF(MP_QSTR_d_colon_8, (const byte*)"\x03\x03" "d:8")
QDEF(MP_QSTR_e, (const byte*)"\xc0\x01" "e")
QDEF(MP_QSTR_e3_colon_3, (const byte*)"\xba\x04" "e3:3")
QDEF(MP_QSTR_e4, (const byte*)"\xf4\x02" "e4")
QDEF(MP_QSTR_e4_colon_1, (const byte*)"\x3f\x04" "e4:1")
QDEF(MP_QSTR_e5, (const byte*)"\xf5\x02" "e5")
QDEF(MP_QSTR_e6_colon_3, (const byte*)"\x3f\x04" "e6:3")
QDEF(MP_QSTR_e_colon_1, (const byte*)"\x0b\x03" "e:1")
QDEF(MP_QSTR_e_colon_2, (const byte*)"\x08\x03" "e:2")
QDEF(MP_QSTR_e_colon_3, (const byte*)"\x09\x03" "e:3")
QDEF(MP_QSTR_e_colon_4, (const byte*)"\x0e\x03" "e:4")
QDEF(MP_QSTR_e_colon_5, (const byte*)"\x0f\x03" "e:5")
QDEF(MP_QSTR_e_colon_6, (const byte*)"\x0c\x03" "e:6")
QDEF(MP_QSTR_e_colon_8, (const byte*)"\x02\x03" "e:8")
QDEF(MP_QSTR_eb_colon_8, (const byte*)"\xe0\x04" "eb:8")
QDEF(MP_QSTR_f, (const byte*)"\xc3\x01" "f")
QDEF(MP_QSTR_f_hash_, (const byte*)"\x01\x02" "f#")
QDEF(MP_QSTR_f_hash_5, (const byte*)"\x35\x03" "f#5")
QDEF(MP_QSTR_f_hash_5_colon_2, (const byte*)"\xfd\x05" "f#5:2")
QDEF(MP_QSTR_f_hash__colon_1, (const byte*)"\x4b\x04" "f#:1")
QDEF(MP_QSTR_f_hash__colon_2, (const byte*)"\x48\x04" "f#:2")
QDEF(MP_QSTR_f_hash__colon_8, (const byte*)"\x42\x04" "f#:8")
QDEF(MP_QSTR_f2, (const byte*)"\x11\x02" "f2")
QDEF(MP_QSTR_f_colon_1, (const byte*)"\x08\x03" "f:1")
QDEF(MP_QSTR_f_colon_2, (const byte*)"\x0b\x03" "f:2")
QDEF(MP_QSTR_f_colon_3, (const byte*)"\x0a\x03" "f:3")
QDEF(MP_QSTR_f_colon_4, (const byte*)"\x0d\x03" "f:4")
QDEF(MP_QSTR_f_colon_8, (const byte*)"\x01\x03" "f:8")
QDEF(MP_QSTR_g, (const byte*)"\xc2\x01" "g")
QDEF(MP_QSTR_g_hash_, (const byte*)"\x21\x02" "g#")
QDEF(MP_QSTR_g_hash__colon_1, (const byte*)"\xea\x04" "g#:1")
QDEF(MP_QSTR_g_hash__colon_3, (const byte*)"\xe8\x04" "g#:3")
QDEF(MP_QSTR_g3_colon_1, (const byte*)"\xfa\x04" "g3:1")
QDEF(MP_QSTR_g4, (const byte*)"\x36\x02" "g4")
QDEF(MP_QSTR_g4_colon_1, (const byte*)"\x7d\x04" "g4:1")
QDEF(MP_QSTR_g4_colon_2, (const byte*)"\x7e\x04" "g4:2")
QDEF(MP_QSTR_g5, (const byte*)"\x37\x02" "g5")
QDEF(MP_QSTR_g5_colon_1, (const byte*)"\xfc\x04" "g5:1")
QDEF(MP_QSTR_g_colon_1, (const byte*)"\x09\x03" "g:1")
QDEF(MP_QSTR_g_colon_2, (const byte*)"\x0a\x03" "g:2")
QDEF(MP_QSTR_g_colon_3, (const byte*)"\x0b\x03" "g:3")
QDEF(MP_QSTR_g_colon_8, (const byte*)"\x01\x03" "g:8")
QDEF(MP_QSTR_r, (const byte*)"\xd7\x01" "r")
QDEF(MP_QSTR_r4_colon_2, (const byte*)"\xcb\x04" "r4:2")
QDEF(MP_QSTR_r_colon_1, (const byte*)"\x1c\x03" "r:1")
QDEF(MP_QSTR_r_colon_2, (const byte*)"\x1f\x03" "r:2")
QDEF(MP_QSTR_r_colon_3, (const byte*)"\x1e\x03" "r:3")
QDEF(MP_QSTR_MicroBitUART, (const byte*)"\x32\x0c" "MicroBitUART")
QDEF(MP_QSTR_uart, (const byte*)"\x77\x04" "uart")
QDEF(MP_QSTR_init, (const byte*)"\x1f\x04" "init")
QDEF(MP_QSTR_baudrate, (const byte*)"\xf5\x08" "baudrate")
QDEF(MP_QSTR_bits, (const byte*)"\x49\x04" "bits")
QDEF(MP_QSTR_parity, (const byte*)"\x42\x06" "parity")
QDEF(MP_QSTR_stop, (const byte*)"\x9d\x04" "stop")
QDEF(MP_QSTR_pins, (const byte*)"\x41\x04" "pins")
QDEF(MP_QSTR_tx, (const byte*)"\x89\x02" "tx")
QDEF(MP_QSTR_rx, (const byte*)"\xcf\x02" "rx")
QDEF(MP_QSTR_any, (const byte*)"\x13\x03" "any")
QDEF(MP_QSTR_readall, (const byte*)"\x76\x07" "readall")
QDEF(MP_QSTR_readline, (const byte*)"\xf9\x08" "readline")
QDEF(MP_QSTR_readinto, (const byte*)"\x4b\x08" "readinto")
QDEF(MP_QSTR_ODD, (const byte*)"\x6a\x03" "ODD")
QDEF(MP_QSTR_EVEN, (const byte*)"\xdd\x04" "EVEN")
QDEF(MP_QSTR_MicroBitSPI, (const byte*)"\x4a\x0b" "MicroBitSPI")
QDEF(MP_QSTR_spi, (const byte*)"\xcf\x03" "spi")
QDEF(MP_QSTR_mode, (const byte*)"\x26\x04" "mode")
QDEF(MP_QSTR_sclk, (const byte*)"\x72\x04" "sclk")
QDEF(MP_QSTR_mosi, (const byte*)"\x1d\x04" "mosi")
QDEF(MP_QSTR_miso, (const byte*)"\x9d\x04" "miso")
QDEF(MP_QSTR_write_readinto, (const byte*)"\x89\x0e" "write_readinto")
QDEF(MP_QSTR_neopixel, (const byte*)"\x69\x08" "neopixel")
QDEF(MP_QSTR_NeoPixel, (const byte*)"\x69\x08" "NeoPixel")
QDEF(MP_QSTR_clear, (const byte*)"\x7c\x05" "clear")
QDEF(MP_QSTR_random, (const byte*)"\xbe\x06" "random")
QDEF(MP_QSTR_getrandbits, (const byte*)"\x66\x0b" "getrandbits")
QDEF(MP_QSTR_seed, (const byte*)"\x92\x04" "seed")
QDEF(MP_QSTR_randrange, (const byte*)"\xa3\x09" "randrange")
QDEF(MP_QSTR_randint, (const byte*)"\xaf\x07" "randint")
QDEF(MP_QSTR_choice, (const byte*)"\x2e\x06" "choice")
QDEF(MP_QSTR_uniform, (const byte*)"\x01\x07" "uniform")
QDEF(MP_QSTR_audio, (const byte*)"\x53\x05" "audio")
QDEF(MP_QSTR_AudioFrame, (const byte*)"\xae\x0a" "AudioFrame")
QDEF(MP_QSTR_return_pin, (const byte*)"\x27\x0a" "return_pin")
QDEF(MP_QSTR_source, (const byte*)"\xb8\x06" "source")
QDEF(MP_QSTR_copyfrom, (const byte*)"\x56\x08" "copyfrom")
QDEF(MP_QSTR_name, (const byte*)"\xa2\x04" "name")
QDEF(MP_QSTR_os, (const byte*)"\x79\x02" "os")
QDEF(MP_QSTR_uname, (const byte*)"\xb7\x05" "uname")
QDEF(MP_QSTR_micropython, (const byte*)"\x0b\x0b" "micropython")
QDEF(MP_QSTR_sysname, (const byte*)"\x9b\x07" "sysname")
QDEF(MP_QSTR_nodename, (const byte*)"\x62\x08" "nodename")
QDEF(MP_QSTR_release, (const byte*)"\xec\x07" "release")
QDEF(MP_QSTR_version, (const byte*)"\xbf\x07" "version")
QDEF(MP_QSTR_BytesIO, (const byte*)"\x1a\x07" "BytesIO")
QDEF(MP_QSTR_TextIO, (const byte*)"\x1e\x06" "TextIO")
QDEF(MP_QSTR_writable, (const byte*)"\xf7\x08" "writable")
QDEF(MP_QSTR_listdir, (const byte*)"\x98\x07" "listdir")
QDEF(MP_QSTR_machine, (const byte*)"\x60\x07" "machine")
QDEF(MP_QSTR_size, (const byte*)"\x20\x04" "size")
QDEF(MP_QSTR_is_playing, (const byte*)"\x04\x0a" "is_playing")
QDEF(MP_QSTR_speech, (const byte*)"\x6d\x06" "speech")
QDEF(MP_QSTR_say, (const byte*)"\xae\x03" "say")
QDEF(MP_QSTR_pronounce, (const byte*)"\x94\x09" "pronounce")
QDEF(MP_QSTR_sing, (const byte*)"\xb6\x04" "sing")
QDEF(MP_QSTR_throat, (const byte*)"\x31\x06" "throat")
QDEF(MP_QSTR_mouth, (const byte*)"\x6e\x05" "mouth")
QDEF(MP_QSTR_speed, (const byte*)"\x62\x05" "speed")
QDEF(MP_QSTR_debug, (const byte*)"\xd4\x05" "debug")
QDEF(MP_QSTR_translate, (const byte*)"\x43\x09" "translate")
QDEF(MP_QSTR_radio, (const byte*)"\xd4\x05" "radio")
QDEF(MP_QSTR_config, (const byte*)"\x4f\x06" "config")
QDEF(MP_QSTR_send_bytes, (const byte*)"\xbf\x0a" "send_bytes")
QDEF(MP_QSTR_receive_bytes, (const byte*)"\x88\x0d" "receive_bytes")
QDEF(MP_QSTR_send, (const byte*)"\xb9\x04" "send")
QDEF(MP_QSTR_receive, (const byte*)"\x4e\x07" "receive")
QDEF(MP_QSTR_receive_bytes_into, (const byte*)"\x6b\x12" "receive_bytes_into")
QDEF(MP_QSTR_receive_full, (const byte*)"\x02\x0c" "receive_full")
QDEF(MP_QSTR_length, (const byte*)"\x59\x06" "length")
QDEF(MP_QSTR_queue, (const byte*)"\x94\x05" "queue")
QDEF(MP_QSTR_channel, (const byte*)"\x26\x07" "channel")
QDEF(MP_QSTR_power, (const byte*)"\xda\x05" "power")
QDEF(MP_QSTR_data_rate, (const byte*)"\xa8\x09" "data_rate")
QDEF(MP_QSTR_address, (const byte*)"\x73\x07" "address")
QDEF(MP_QSTR_group, (const byte*)"\xba\x05" "group")
QDEF(MP_QSTR_RATE_250KBIT, (const byte*)"\x7b\x0c" "RATE_250KBIT")
QDEF(MP_QSTR_RATE_1MBIT, (const byte*)"\xdb\x0a" "RATE_1MBIT")
QDEF(MP_QSTR_RATE_2MBIT, (const byte*)"\x58\x0a" "RATE_2MBIT")
QDEF(MP_QSTR_ArithmeticError, (const byte*)"\x2d\x0f" "ArithmeticError")
QDEF(MP_QSTR_AssertionError, (const byte*)"\x97\x0e" "AssertionError")
QDEF(MP_QSTR_AttributeError, (const byte*)"\x21\x0e" "AttributeError")
QDEF(MP_QSTR_BaseException, (const byte*)"\x07\x0d" "BaseException")
QDEF(MP_QSTR_EOFError, (const byte*)"\x91\x08" "EOFError")
QDEF(MP_QSTR_Ellipsis, (const byte*)"\xf0\x08" "Ellipsis")
QDEF(MP_QSTR_Exception, (const byte*)"\xf2\x09" "Exception")
QDEF(MP_QSTR_GeneratorExit, (const byte*)"\x16\x0d" "GeneratorExit")
QDEF(MP_QSTR_ImportError, (const byte*)"\x20\x0b" "ImportError")
QDEF(MP_QSTR_IndentationError, (const byte*)"\x5c\x10" "IndentationError")
QDEF(MP_QSTR_IndexError, (const byte*)"\x83\x0a" "IndexError")
QDEF(MP_QSTR_KeyError, (const byte*)"\xea\x08" "KeyError")
QDEF(MP_QSTR_KeyboardInterrupt, (const byte*)"\xaf\x11" "KeyboardInterrupt")
QDEF(MP_QSTR_LookupError, (const byte*)"\xff\x0b" "LookupError")
QDEF(MP_QSTR_MemoryError, (const byte*)"\xdc\x0b" "MemoryError")
QDEF(MP_QSTR_NameError, (const byte*)"\xba\x09" "NameError")
QDEF(MP_QSTR_NoneType, (const byte*)"\x17\x08" "NoneType")
QDEF(MP_QSTR_NotImplementedError, (const byte*)"\xc6\x13" "NotImplementedError")
QDEF(MP_QSTR_OSError, (const byte*)"\xa1\x07" "OSError")
QDEF(MP_QSTR_OrderedDict, (const byte*)"\xf0\x0b" "OrderedDict")
QDEF(MP_QSTR_OverflowError, (const byte*)"\x81\x0d" "OverflowError")
QDEF(MP_QSTR_RuntimeError, (const byte*)"\x61\x0c" "RuntimeError")
QDEF(MP_QSTR_StopIteration, (const byte*)"\xea\x0d" "StopIteration")
QDEF(MP_QSTR_SyntaxError, (const byte*)"\x94\x0b" "SyntaxError")
QDEF(MP_QSTR_SystemExit, (const byte*)"\x20\x0a" "SystemExit")
QDEF(MP_QSTR_TypeError, (const byte*)"\x25\x09" "TypeError")
QDEF(MP_QSTR_UnicodeError, (const byte*)"\x22\x0c" "UnicodeError")
QDEF(MP_QSTR_ValueError, (const byte*)"\x96\x0a" "ValueError")
QDEF(MP_QSTR_ZeroDivisionError, (const byte*)"\xb6\x11" "ZeroDivisionError")
QDEF(MP_QSTR___add__, (const byte*)"\xc4\x07" "__add__")
QDEF(MP_QSTR___bool__, (const byte*)"\x2b\x08" "__bool__")
QDEF(MP_QSTR___build_class__, (const byte*)"\x42\x0f" "__build_class__")
QDEF(MP_QSTR___call__, (const byte*)"\xa7\x08" "__call__")
QDEF(MP_QSTR___class__, (const byte*)"\x2b\x09" "__class__")
QDEF(MP_QSTR___contains__, (const byte*)"\xc6\x0c" "__contains__")
QDEF(MP_QSTR___delitem__, (const byte*)"\xfd\x0b" "__delitem__")
QDEF(MP_QSTR___enter__, (const byte*)"\x6d\x09" "__enter__")
QDEF(MP_QSTR___eq__, (const byte*)"\x71\x06" "__eq__")
QDEF(MP_QSTR___exit__, (const byte*)"\x45\x08" "__exit__")
QDEF(MP_QSTR___ge__, (const byte*)"\xa7\x06" "__ge__")
QDEF(MP_QSTR___getattr__, (const byte*)"\x40\x0b" "__getattr__")
QDEF(MP_QSTR___getitem__, (const byte*)"\x26\x0b" "__getitem__")
QDEF(MP_QSTR___gt__, (const byte*)"\xb6\x06" "__gt__")
QDEF(MP_QSTR___hash__, (const byte*)"\xf7\x08" "__hash__")
QDEF(MP_QSTR___import__, (const byte*)"\x38\x0a" "__import__")
QDEF(MP_QSTR___init__, (const byte*)"\x5f\x08" "__init__")
QDEF(MP_QSTR___iter__, (const byte*)"\xcf\x08" "__iter__")
QDEF(MP_QSTR___le__, (const byte*)"\xcc\x06" "__le__")
QDEF(MP_QSTR___len__, (const byte*)"\xe2\x07" "__len__")
QDEF(MP_QSTR___lt__, (const byte*)"\x5d\x06" "__lt__")
QDEF(MP_QSTR___main__, (const byte*)"\x8e\x08" "__main__")
QDEF(MP_QSTR___module__, (const byte*)"\xff\x0a" "__module__")
QDEF(MP_QSTR___name__, (const byte*)"\xe2\x08" "__name__")
QDEF(MP_QSTR___new__, (const byte*)"\x79\x07" "__new__")
QDEF(MP_QSTR___next__, (const byte*)"\x02\x08" "__next__")
QDEF(MP_QSTR___path__, (const byte*)"\xc8\x08" "__path__")
QDEF(MP_QSTR___qualname__, (const byte*)"\x6b\x0c" "__qualname__")
QDEF(MP_QSTR___repl_print__, (const byte*)"\x01\x0e" "__repl_print__")
QDEF(MP_QSTR___repr__, (const byte*)"\x10\x08" "__repr__")
QDEF(MP_QSTR___reversed__, (const byte*)"\x61\x0c" "__reversed__")
QDEF(MP_QSTR___setitem__, (const byte*)"\x32\x0b" "__setitem__")
QDEF(MP_QSTR___str__, (const byte*)"\xd0\x07" "__str__")
QDEF(MP_QSTR___sub__, (const byte*)"\x21\x07" "__sub__")
QDEF(MP_QSTR___traceback__, (const byte*)"\x4f\x0d" "__traceback__")
QDEF(MP_QSTR_abs, (const byte*)"\x95\x03" "abs")
QDEF(MP_QSTR_acos, (const byte*)"\x1b\x04" "acos")
QDEF(MP_QSTR_add, (const byte*)"\x44\x03" "add")
QDEF(MP_QSTR_align, (const byte*)"\xa8\x05" "align")
QDEF(MP_QSTR_all, (const byte*)"\x44\x03" "all")
QDEF(MP_QSTR_and_, (const byte*)"\x91\x04" "and_")
QDEF(MP_QSTR_append, (const byte*)"\x6b\x06" "append")
QDEF(MP_QSTR_args, (const byte*)"\xc2\x04" "args")
QDEF(MP_QSTR_array, (const byte*)"\x7c\x05" "array")
QDEF(MP_QSTR_asin, (const byte*)"\x50\x04" "asin")
QDEF(MP_QSTR_asm_thumb, (const byte*)"\x43\x09" "asm_thumb")
QDEF(MP_QSTR_asr, (const byte*)"\x65\x03" "asr")
QDEF(MP_QSTR_atan, (const byte*)"\x1f\x04" "atan")
QDEF(MP_QSTR_atan2, (const byte*)"\xcd\x05" "atan2")
QDEF(MP_QSTR_bin, (const byte*)"\xe0\x03" "bin")
QDEF(MP_QSTR_bl, (const byte*)"\xcb\x02" "bl")
QDEF(MP_QSTR_bool, (const byte*)"\xeb\x04" "bool")
QDEF(MP_QSTR_bound_method, (const byte*)"\x97\x0c" "bound_method")
QDEF(MP_QSTR_builtins, (const byte*)"\xf7\x08" "builtins")
QDEF(MP_QSTR_bx, (const byte*)"\xdf\x02" "bx")
QDEF(MP_QSTR_bytearray, (const byte*)"\x76\x09" "bytearray")
QDEF(MP_QSTR_bytecode, (const byte*)"\x22\x08" "bytecode")
QDEF(MP_QSTR_byteorder, (const byte*)"\x61\x09" "byteorder")
QDEF(MP_QSTR_bytes, (const byte*)"\x5c\x05" "bytes")
QDEF(MP_QSTR_calcsize, (const byte*)"\x4d\x08" "calcsize")
QDEF(MP_QSTR_callable, (const byte*)"\x0d\x08" "callable")
QDEF(MP_QSTR_ceil, (const byte*)"\x06\x04" "ceil")
QDEF(MP_QSTR_chr, (const byte*)"\xdc\x03" "chr")
QDEF(MP_QSTR_classmethod, (const byte*)"\xb4\x0b" "classmethod")
QDEF(MP_QSTR_close, (const byte*)"\x33\x05" "close")
QDEF(MP_QSTR_closure, (const byte*)"\x74\x07" "closure")
QDEF(MP_QSTR_clz, (const byte*)"\x50\x03" "clz")
QDEF(MP_QSTR_cmp, (const byte*)"\x3b\x03" "cmp")
QDEF(MP_QSTR_collect, (const byte*)"\x9b\x07" "collect")
QDEF(MP_QSTR_const, (const byte*)"\xc0\x05" "const")
QDEF(MP_QSTR_copysign, (const byte*)"\x33\x08" "copysign")
QDEF(MP_QSTR_cos, (const byte*)"\x7a\x03" "cos")
QDEF(MP_QSTR_count, (const byte*)"\xa6\x05" "count")
QDEF(MP_QSTR_cpsid, (const byte*)"\xe8\x05" "cpsid")
QDEF(MP_QSTR_cpsie, (const byte*)"\xe9\x05" "cpsie")
QDEF(MP_QSTR_data, (const byte*)"\x15\x04" "data")
QDEF(MP_QSTR_default, (const byte*)"\xce\x07" "default")
QDEF(MP_QSTR_degrees, (const byte*)"\x02\x07" "degrees")
QDEF(MP_QSTR_dict, (const byte*)"\x3f\x04" "dict")
QDEF(MP_QSTR_dict_view, (const byte*)"\x2d\x09" "dict_view")
QDEF(MP_QSTR_difference, (const byte*)"\x72\x0a" "difference")
QDEF(MP_QSTR_difference_update, (const byte*)"\x9c\x11" "difference_update")
QDEF(MP_QSTR_dir, (const byte*)"\xfa\x03" "dir")
QDEF(MP_QSTR_disable, (const byte*)"\x91\x07" "disable")
QDEF(MP_QSTR_disable_irq, (const byte*)"\x04\x0b" "disable_irq")
QDEF(MP_QSTR_discard, (const byte*)"\x0f\x07" "discard")
QDEF(MP_QSTR_divmod, (const byte*)"\xb8\x06" "divmod")
QDEF(MP_QSTR_enable, (const byte*)"\x04\x06" "enable")
QDEF(MP_QSTR_enable_irq, (const byte*)"\x91\x0a" "enable_irq")
QDEF(MP_QSTR_end, (const byte*)"\x0a\x03" "end")
QDEF(MP_QSTR_endswith, (const byte*)"\x1b\x08" "endswith")
QDEF(MP_QSTR_enumerate, (const byte*)"\x71\x09" "enumerate")
QDEF(MP_QSTR_eval, (const byte*)"\x9b\x04" "eval")
QDEF(MP_QSTR_exec, (const byte*)"\x1e\x04" "exec")
QDEF(MP_QSTR_exit, (const byte*)"\x85\x04" "exit")
QDEF(MP_QSTR_exp, (const byte*)"\xc8\x03" "exp")
QDEF(MP_QSTR_extend, (const byte*)"\x63\x06" "extend")
QDEF(MP_QSTR_fabs, (const byte*)"\x93\x04" "fabs")
QDEF(MP_QSTR_filter, (const byte*)"\x25\x06" "filter")
QDEF(MP_QSTR_find, (const byte*)"\x01\x04" "find")
QDEF(MP_QSTR_float, (const byte*)"\x35\x05" "float")
QDEF(MP_QSTR_floor, (const byte*)"\x7d\x05" "floor")
QDEF(MP_QSTR_fmod, (const byte*)"\xe5\x04" "fmod")
QDEF(MP_QSTR_format, (const byte*)"\x26\x06" "format")
QDEF(MP_QSTR_frexp, (const byte*)"\x1c\x05" "frexp")
QDEF(MP_QSTR_from_bytes, (const byte*)"\x35\x0a" "from_bytes")
QDEF(MP_QSTR_fromkeys, (const byte*)"\x37\x08" "fromkeys")
QDEF(MP_QSTR_frozenset, (const byte*)"\xed\x09" "frozenset")
QDEF(MP_QSTR_function, (const byte*)"\x27\x08" "function")
QDEF(MP_QSTR_gc, (const byte*)"\x61\x02" "gc")
QDEF(MP_QSTR_generator, (const byte*)"\x96\x09" "generator")
QDEF(MP_QSTR_get, (const byte*)"\x33\x03" "get")
QDEF(MP_QSTR_getattr, (const byte*)"\xc0\x07" "getattr")
QDEF(MP_QSTR_globals, (const byte*)"\x9d\x07" "globals")
QDEF(MP_QSTR_hasattr, (const byte*)"\x8c\x07" "hasattr")
QDEF(MP_QSTR_hash, (const byte*)"\xb7\x04" "hash")
QDEF(MP_QSTR_heap_lock, (const byte*)"\xad\x09" "heap_lock")
QDEF(MP_QSTR_heap_unlock, (const byte*)"\x56\x0b" "heap_unlock")
QDEF(MP_QSTR_hex, (const byte*)"\x70\x03" "hex")
QDEF(MP_QSTR_id, (const byte*)"\x28\x02" "id")
QDEF(MP_QSTR_implementation, (const byte*)"\x17\x0e" "implementation")
QDEF(MP_QSTR_index, (const byte*)"\x7b\x05" "index")
QDEF(MP_QSTR_insert, (const byte*)"\x12\x06" "insert")
QDEF(MP_QSTR_int, (const byte*)"\x16\x03" "int")
QDEF(MP_QSTR_intersection, (const byte*)"\x28\x0c" "intersection")
QDEF(MP_QSTR_intersection_update, (const byte*)"\x06\x13" "intersection_update")
QDEF(MP_QSTR_isalpha, (const byte*)"\xeb\x07" "isalpha")
QDEF(MP_QSTR_isdigit, (const byte*)"\xa8\x07" "isdigit")
QDEF(MP_QSTR_isdisjoint, (const byte*)"\xf7\x0a" "isdisjoint")
QDEF(MP_QSTR_isenabled, (const byte*)"\x9a\x09" "isenabled")
QDEF(MP_QSTR_isfinite, (const byte*)"\xa6\x08" "isfinite")
QDEF(MP_QSTR_isinf, (const byte*)"\x3e\x05" "isinf")
QDEF(MP_QSTR_isinstance, (const byte*)"\xb6\x0a" "isinstance")
QDEF(MP_QSTR_islower, (const byte*)"\xfc\x07" "islower")
QDEF(MP_QSTR_isnan, (const byte*)"\x9e\x05" "isnan")
QDEF(MP_QSTR_isspace, (const byte*)"\x5b\x07" "isspace")
QDEF(MP_QSTR_issubclass, (const byte*)"\xb5\x0a" "issubclass")
QDEF(MP_QSTR_issubset, (const byte*)"\xb9\x08" "issubset")
QDEF(MP_QSTR_issuperset, (const byte*)"\xfc\x0a" "issuperset")
QDEF(MP_QSTR_isupper, (const byte*)"\xdd\x07" "isupper")
QDEF(MP_QSTR_items, (const byte*)"\xe3\x05" "items")
QDEF(MP_QSTR_iter, (const byte*)"\x8f\x04" "iter")
QDEF(MP_QSTR_iterator, (const byte*)"\x47\x08" "iterator")
QDEF(MP_QSTR_join, (const byte*)"\xa7\x04" "join")
QDEF(MP_QSTR_kbd_intr, (const byte*)"\xf6\x08" "kbd_intr")
QDEF(MP_QSTR_key, (const byte*)"\x32\x03" "key")
QDEF(MP_QSTR_keys, (const byte*)"\x01\x04" "keys")
QDEF(MP_QSTR_label, (const byte*)"\x43\x05" "label")
QDEF(MP_QSTR_ldexp, (const byte*)"\x40\x05" "ldexp")
QDEF(MP_QSTR_ldr, (const byte*)"\x5f\x03" "ldr")
QDEF(MP_QSTR_ldrb, (const byte*)"\x5d\x04" "ldrb")
QDEF(MP_QSTR_ldrex, (const byte*)"\xe2\x05" "ldrex")
QDEF(MP_QSTR_ldrh, (const byte*)"\x57\x04" "ldrh")
QDEF(MP_QSTR_len, (const byte*)"\x62\x03" "len")
QDEF(MP_QSTR_list, (const byte*)"\x27\x04" "list")
QDEF(MP_QSTR_little, (const byte*)"\x89\x06" "little")
QDEF(MP_QSTR_locals, (const byte*)"\x3b\x06" "locals")
QDEF(MP_QSTR_log, (const byte*)"\x21\x03" "log")
QDEF(MP_QSTR_lower, (const byte*)"\xc6\x05" "lower")
QDEF(MP_QSTR_lsl, (const byte*)"\xb6\x03" "lsl")
QDEF(MP_QSTR_lsr, (const byte*)"\xa8\x03" "lsr")
QDEF(MP_QSTR_lstrip, (const byte*)"\xe5\x06" "lstrip")
QDEF(MP_QSTR_map, (const byte*)"\xb9\x03" "map")
QDEF(MP_QSTR_math, (const byte*)"\x35\x04" "math")
QDEF(MP_QSTR_max, (const byte*)"\xb1\x03" "max")
QDEF(MP_QSTR_mem, (const byte*)"\x20\x03" "mem")
QDEF(MP_QSTR_mem16, (const byte*)"\x07\x05" "mem16")
QDEF(MP_QSTR_mem32, (const byte*)"\x41\x05" "mem32")
QDEF(MP_QSTR_mem8, (const byte*)"\x18\x04" "mem8")
QDEF(MP_QSTR_mem_alloc, (const byte*)"\x52\x09" "mem_alloc")
QDEF(MP_QSTR_mem_free, (const byte*)"\xcb\x08" "mem_free")
QDEF(MP_QSTR_mem_info, (const byte*)"\xd1\x08" "mem_info")
QDEF(MP_QSTR_min, (const byte*)"\xaf\x03" "min")
QDEF(MP_QSTR_modf, (const byte*)"\x25\x04" "modf")
QDEF(MP_QSTR_module, (const byte*)"\xbf\x06" "module")
QDEF(MP_QSTR_modules, (const byte*)"\xec\x07" "modules")
QDEF(MP_QSTR_mov, (const byte*)"\xf1\x03" "mov")
QDEF(MP_QSTR_movt, (const byte*)"\x65\x04" "movt")
QDEF(MP_QSTR_movw, (const byte*)"\x66\x04" "movw")
QDEF(MP_QSTR_movwt, (const byte*)"\x52\x05" "movwt")
QDEF(MP_QSTR_mrs, (const byte*)"\x89\x03" "mrs")
QDEF(MP_QSTR_namedtuple, (const byte*)"\x1e\x0a" "namedtuple")
QDEF(MP_QSTR_next, (const byte*)"\x42\x04" "next")
QDEF(MP_QSTR_nop, (const byte*)"\xb4\x03" "nop")
QDEF(MP_QSTR_object, (const byte*)"\x90\x06" "object")
QDEF(MP_QSTR_oct, (const byte*)"\xfd\x03" "oct")
QDEF(MP_QSTR_open, (const byte*)"\xd1\x04" "open")
QDEF(MP_QSTR_opt_level, (const byte*)"\x87\x09" "opt_level")
QDEF(MP_QSTR_ord, (const byte*)"\x1c\x03" "ord")
QDEF(MP_QSTR_pack, (const byte*)"\xbc\x04" "pack")
QDEF(MP_QSTR_pack_into, (const byte*)"\x1f\x09" "pack_into")
QDEF(MP_QSTR_pi, (const byte*)"\x1c\x02" "pi")
QDEF(MP_QSTR_platform, (const byte*)"\x3a\x08" "platform")
QDEF(MP_QSTR_pop, (const byte*)"\x2a\x03" "pop")
QDEF(MP_QSTR_popitem, (const byte*)"\xbf\x07" "popitem")
QDEF(MP_QSTR_pow, (const byte*)"\x2d\x03" "pow")
QDEF(MP_QSTR_print, (const byte*)"\x54\x05" "print")
QDEF(MP_QSTR_print_exception, (const byte*)"\x1c\x0f" "print_exception")
QDEF(MP_QSTR_push, (const byte*)"\xbb\x04" "push")
QDEF(MP_QSTR_qstr_info, (const byte*)"\xb0\x09" "qstr_info")
QDEF(MP_QSTR_radians, (const byte*)"\x87\x07" "radians")
QDEF(MP_QSTR_range, (const byte*)"\x1a\x05" "range")
QDEF(MP_QSTR_rbit, (const byte*)"\xe8\x04" "rbit")
QDEF(MP_QSTR_read_light_level, (const byte*)"\x5f\x10" "read_light_level")
QDEF(MP_QSTR_remove, (const byte*)"\x63\x06" "remove")
QDEF(MP_QSTR_replace, (const byte*)"\x49\x07" "replace")
QDEF(MP_QSTR_repr, (const byte*)"\xd0\x04" "repr")
QDEF(MP_QSTR_reverse, (const byte*)"\x25\x07" "reverse")
QDEF(MP_QSTR_reversed, (const byte*)"\xa1\x08" "reversed")
QDEF(MP_QSTR_rfind, (const byte*)"\xd2\x05" "rfind")
QDEF(MP_QSTR_rindex, (const byte*)"\xe9\x06" "rindex")
QDEF(MP_QSTR_round, (const byte*)"\xe7\x05" "round")
QDEF(MP_QSTR_rsplit, (const byte*)"\xa5\x06" "rsplit")
QDEF(MP_QSTR_rstrip, (const byte*)"\x3b\x06" "rstrip")
QDEF(MP_QSTR_scan, (const byte*)"\x1a\x04" "scan")
QDEF(MP_QSTR_sdiv, (const byte*)"\xcd\x04" "sdiv")
QDEF(MP_QSTR_sep, (const byte*)"\x23\x03" "sep")
QDEF(MP_QSTR_set, (const byte*)"\x27\x03" "set")
QDEF(MP_QSTR_setattr, (const byte*)"\xd4\x07" "setattr")
QDEF(MP_QSTR_setdefault, (const byte*)"\x6c\x0a" "setdefault")
QDEF(MP_QSTR_sin, (const byte*)"\xb1\x03" "sin")
QDEF(MP_QSTR_sleep_ms, (const byte*)"\x0b\x08" "sleep_ms")
QDEF(MP_QSTR_sleep_us, (const byte*)"\x13\x08" "sleep_us")
QDEF(MP_QSTR_sort, (const byte*)"\xbf\x04" "sort")
QDEF(MP_QSTR_sorted, (const byte*)"\x5e\x06" "sorted")
QDEF(MP_QSTR_split, (const byte*)"\xb7\x05" "split")
QDEF(MP_QSTR_sqrt, (const byte*)"\x21\x04" "sqrt")
QDEF(MP_QSTR_stack_use, (const byte*)"\x97\x09" "stack_use")
QDEF(MP_QSTR_startswith, (const byte*)"\x74\x0a" "startswith")
QDEF(MP_QSTR_staticmethod, (const byte*)"\x62\x0c" "staticmethod")
QDEF(MP_QSTR_step, (const byte*)"\x57\x04" "step")
QDEF(MP_QSTR_str, (const byte*)"\x50\x03" "str")
QDEF(MP_QSTR_strb, (const byte*)"\x32\x04" "strb")
QDEF(MP_QSTR_strex, (const byte*)"\xad\x05" "strex")
QDEF(MP_QSTR_strh, (const byte*)"\x38\x04" "strh")
QDEF(MP_QSTR_strip, (const byte*)"\x29\x05" "strip")
QDEF(MP_QSTR_sub, (const byte*)"\x21\x03" "sub")
QDEF(MP_QSTR_sum, (const byte*)"\x2e\x03" "sum")
QDEF(MP_QSTR_super, (const byte*)"\xc4\x05" "super")
QDEF(MP_QSTR_symmetric_difference, (const byte*)"\xce\x14" "symmetric_difference")
QDEF(MP_QSTR_symmetric_difference_update, (const byte*)"\x60\x1b" "symmetric_difference_update")
QDEF(MP_QSTR_sys, (const byte*)"\xbc\x03" "sys")
QDEF(MP_QSTR_tan, (const byte*)"\xfe\x03" "tan")
QDEF(MP_QSTR_threshold, (const byte*)"\xf2\x09" "threshold")
QDEF(MP_QSTR_throw, (const byte*)"\xb3\x05" "throw")
QDEF(MP_QSTR_ticks_add, (const byte*)"\x9d\x09" "ticks_add")
QDEF(MP_QSTR_ticks_diff, (const byte*)"\xb1\x0a" "ticks_diff")
QDEF(MP_QSTR_ticks_ms, (const byte*)"\x42\x08" "ticks_ms")
QDEF(MP_QSTR_ticks_us, (const byte*)"\x5a\x08" "ticks_us")
QDEF(MP_QSTR_time, (const byte*)"\xf0\x04" "time")
QDEF(MP_QSTR_time_pulse_us, (const byte*)"\x89\x0d" "time_pulse_us")
QDEF(MP_QSTR_to_bytes, (const byte*)"\xd8\x08" "to_bytes")
QDEF(MP_QSTR_trunc, (const byte*)"\x5b\x05" "trunc")
QDEF(MP_QSTR_tuple, (const byte*)"\xfd\x05" "tuple")
QDEF(MP_QSTR_type, (const byte*)"\x9d\x04" "type")
QDEF(MP_QSTR_ucollections, (const byte*)"\x15\x0c" "ucollections")
QDEF(MP_QSTR_udiv, (const byte*)"\x8b\x04" "udiv")
QDEF(MP_QSTR_uint, (const byte*)"\xe3\x04" "uint")
QDEF(MP_QSTR_union, (const byte*)"\xf6\x05" "union")
QDEF(MP_QSTR_unique_id, (const byte*)"\x04\x09" "unique_id")
QDEF(MP_QSTR_unpack, (const byte*)"\x07\x06" "unpack")
QDEF(MP_QSTR_unpack_from, (const byte*)"\x0e\x0b" "unpack_from")
QDEF(MP_QSTR_update, (const byte*)"\xb4\x06" "update")
QDEF(MP_QSTR_upper, (const byte*)"\x27\x05" "upper")
QDEF(MP_QSTR_ustruct, (const byte*)"\x47\x07" "ustruct")
QDEF(MP_QSTR_utime, (const byte*)"\xe5\x05" "utime")
QDEF(MP_QSTR_value, (const byte*)"\x4e\x05" "value")
QDEF(MP_QSTR_values, (const byte*)"\x7d\x06" "values")
QDEF(MP_QSTR_version_info, (const byte*)"\x6e\x0c" "version_info")
QDEF(MP_QSTR_wfi, (const byte*)"\x9d\x03" "wfi")
QDEF(MP_QSTR_zip, (const byte*)"\xe6\x03" "zip")
