// This file was generated by py/makeversionhdr.py
#define MICROPY_GIT_TAG "v1.9.2-34-gd64154c73"
#define MICROPY_GIT_HASH "d64154c73"
#define MICROPY_BUILD_DATE "2017-09-01"
#define MICROPY_VERSION_MAJOR (1)
#define MICROPY_VERSION_MINOR (9)
#define MICROPY_VERSION_MICRO (2)
#define MICROPY_VERSION_STRING "1.9.2"
