/**
  * MicroBitCustomConfig.h
  *
  * This file is automatically included by the microbit DAL compilation
  * process. Use this to define any custom configration options needed
  * for your build of the micro:bit runtime.
  *
  * See microbit-dal/inc/MicroBitConfig.h for a complete list of options.
  * Any options you define here will take prescedence over those defined there.
  */

#ifndef MICROBIT_CUSTOM_CONFIG_H
#define MICROBIT_CUSTOM_CONFIG_H

#define MICROBIT_HEAP_REUSE_SD      0
#define MICROBIT_BLE_ENABLED        0
#define MICROBIT_BLE_BLUEZONE       0
#define MICROBIT_BLE_DFU_SERVICE    0
#define MICROBIT_BLE_EVENT_SERVICE  0
#define MICROBIT_BLE_DEVICE_INFORMATION_SERVICE     0
#define MICROBIT_BLE_PAIRING_MODE   0
#define MICROBIT_RADIO_ENABLED      0

#endif
